@@ -59,6 +59,7 @@ struct _GdaSqliteRecordsetPrivate {
 	GdaRow       *tmp_row; /* used in cursor mode */
 };
 static GObjectClass *parent_class = NULL;
+GHashTable *error_blobs_hash = NULL;
 
 /*
  * Object init and finalize
@@ -88,6 +89,9 @@ gda_sqlite_recordset_class_init (GdaSqliteRecordsetClass *klass)
 	pmodel_class->fetch_next = gda_sqlite_recordset_fetch_next;
 	pmodel_class->fetch_prev = NULL;
 	pmodel_class->fetch_at = NULL;
+
+	g_assert (!error_blobs_hash);
+	error_blobs_hash = g_hash_table_new (NULL, NULL);
 }
 
 static void
@@ -156,17 +160,19 @@ read_rows_to_init_col_types (GdaSqliteRecordset *model)
 		if (pmodel->prep_stmt->types[i] == GDA_TYPE_NULL)
 			missing_cols [nb_missing++] = i;
 	}
-	/*
+
+#ifdef GDA_DEBUG_NO
 	if (nb_missing == 0)
-		g_print ("Hey!, all columns are known for prep stmt %p\n", pmodel->prep_stmt);
-	*/
+		g_print ("All columns are known for model %p\n", pmodel);
+#endif
+
 	for (; nb_missing > 0; ) {
 		GdaRow *prow;
 		prow = fetch_next_sqlite_row (model, TRUE, NULL);
 		if (!prow)
 			break;
 #ifdef GDA_DEBUG_NO
-		g_print ("Read row %d for prep stmt %p\n", model->priv->next_row_num - 1, pmodel->prep_stmt);
+		g_print ("Prefetched row %d of model %p\n", model->priv->next_row_num - 1, pmodel);
 #endif
 		for (i = nb_missing - 1; i >= 0; i--) {
 			if (pmodel->prep_stmt->types [missing_cols [i]] != GDA_TYPE_NULL) {
@@ -333,7 +339,6 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 	switch (rc) {
 	case  SQLITE_ROW: {
 		gint col, real_col;
-		
 		prow = gda_row_new (_GDA_PSTMT (ps)->ncols);
 		for (col = 0; col < _GDA_PSTMT (ps)->ncols; col++) {
 			GValue *value;
@@ -377,7 +382,14 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 			
 			/* fill GValue */
 			value = gda_row_get_value (prow, col);
-			if (SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, real_col) == NULL) {
+			GError *may_error;
+			may_error = (GError*) SQLITE3_CALL (sqlite3_column_blob) (ps->sqlite_stmt, real_col);
+			if (may_error && g_hash_table_lookup (error_blobs_hash, may_error)) {
+				/*g_print ("Row invalidated: [%s]\n", may_error->message);*/
+				gda_row_invalidate_value_e (prow, value, may_error);
+				g_hash_table_remove (error_blobs_hash, may_error);
+			}
+			else if (SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, real_col) == NULL) {
 				/* we have a NULL value */
 				gda_value_set_null (value);
 			}
@@ -390,10 +402,11 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 					gint64 i;
 					i = SQLITE3_CALL (sqlite3_column_int64) (ps->sqlite_stmt, real_col);
 					if ((i > G_MAXINT) || (i < G_MININT)) {
-						g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
 							     "%s", _("Integer value is too big"));
-						gda_row_invalidate_value (prow, value);
+						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
 						g_value_set_int (value, (gint) i);
@@ -402,10 +415,11 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 					guint64 i;
 					i = (gint64) SQLITE3_CALL (sqlite3_column_int64) (ps->sqlite_stmt, real_col);
 					if (i > G_MAXUINT) {
-						g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
 							     "%s", _("Integer value is too big"));
-						gda_row_invalidate_value (prow, value);
+						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
 						g_value_set_uint (value, (gint) i);
@@ -467,10 +481,11 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 									      rowid);
 					}
 					if (!bop) {
-						g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
 							     "%s", _("Unable to open BLOB"));
-						gda_row_invalidate_value (prow, value);
+						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else {
 						GdaBlob *blob;
@@ -487,11 +502,12 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 					if (!gda_parse_iso8601_date (&date, 
 								     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, 
 												    real_col))) {
-						g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
 							     _("Invalid date '%s' (date format should be YYYY-MM-DD)"), 
 							     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, real_col));
-						gda_row_invalidate_value (prow, value);
+						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
 						g_value_set_boxed (value, &date);
@@ -501,11 +517,12 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 					if (!gda_parse_iso8601_time (&timegda, 
 								     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, 
 												    real_col))) {
-						g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
 							     _("Invalid time '%s' (time format should be HH:MM:SS[.ms])"), 
 							     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, real_col));
-						gda_row_invalidate_value (prow, value);
+						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
 						gda_value_set_time (value, &timegda);
@@ -515,42 +532,76 @@ fetch_next_sqlite_row (GdaSqliteRecordset *model, gboolean do_store, GError **er
 					if (!gda_parse_iso8601_timestamp (&timestamp, 
 									  (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt,
 													 real_col))) {
-						g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
 							     _("Invalid timestamp '%s' (format should be YYYY-MM-DD HH:MM:SS[.ms])"), 
 							     (gchar *) SQLITE3_CALL (sqlite3_column_text) (ps->sqlite_stmt, real_col));
-						gda_row_invalidate_value (prow, value);
+						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
 						gda_value_set_timestamp (value, &timestamp);
 				}
-				else if (type == G_TYPE_INT) {
+				else if (type == G_TYPE_CHAR) {
 					gint64 i;
-					i = SQLITE3_CALL (sqlite3_column_int64) (ps->sqlite_stmt, real_col);
+					i = (gint64) SQLITE3_CALL (sqlite3_column_int64) (ps->sqlite_stmt, real_col);
 					if ((i > G_MAXINT8) || (i < G_MININT8)) {
-						g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
 							     "%s", _("Integer value is too big"));
-						gda_row_invalidate_value (prow, value);
+						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
 						g_value_set_char (value, (gchar) i);
 				}
 				else if (type == G_TYPE_UCHAR) {
-					guint64 i;
+					gint64 i;
 					i = (gint64) SQLITE3_CALL (sqlite3_column_int64) (ps->sqlite_stmt, real_col);
-					if (i > G_MAXUINT8) {
-						g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+					if ((i > G_MAXUINT8) || (i < 0)) {
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
 							     GDA_SERVER_PROVIDER_DATA_ERROR,
 							     "%s", _("Integer value is too big"));
-						gda_row_invalidate_value (prow, value);
+						gda_row_invalidate_value_e (prow, value, lerror);
 					}
 					else
 						g_value_set_uchar (value, (guchar) i);
 				}
-				else 
-					g_error ("Unhandled GDA type %s in SQLite recordset", 
-						 gda_g_type_to_string (_GDA_PSTMT (ps)->types [col]));
+				else if (type == GDA_TYPE_SHORT) {
+					gint64 i;
+					i = (gint64) SQLITE3_CALL (sqlite3_column_int64) (ps->sqlite_stmt, real_col);
+					if ((i > G_MAXSHORT) || (i < G_MINSHORT)) {
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
+							     GDA_SERVER_PROVIDER_DATA_ERROR,
+							     "%s", _("Integer value is too big"));
+						gda_row_invalidate_value_e (prow, value, lerror);
+					}
+					else
+						gda_value_set_short (value, (guchar) i);
+				}
+				else if (type == GDA_TYPE_USHORT) {
+					gint64 i;
+					i = (gint64) SQLITE3_CALL (sqlite3_column_int64) (ps->sqlite_stmt, real_col);
+					if ((i > G_MAXUSHORT) || (i < 0)) {
+						GError *lerror = NULL;
+						g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
+							     GDA_SERVER_PROVIDER_DATA_ERROR,
+							     "%s", _("Integer value is too big"));
+						gda_row_invalidate_value_e (prow, value, lerror);
+					}
+					else
+						gda_value_set_ushort (value, (guchar) i);
+				}
+				else {
+					GError *lerror = NULL;
+					g_set_error (&lerror, GDA_SERVER_PROVIDER_ERROR,
+						     GDA_SERVER_PROVIDER_DATA_ERROR,
+						     "Unhandled type '%s' in SQLite recordset",
+						     gda_g_type_to_string (_GDA_PSTMT (ps)->types [col]));
+					gda_row_invalidate_value_e (prow, value, lerror);
+				}
 			}
 		}
 		
@@ -618,35 +669,24 @@ gda_sqlite_recordset_fetch_nb_rows (GdaDataSelect *model)
 /*
  * Create a new filled #GdaRow object for the row at position @rownum.
  *
- * Each new #GdaRow created is "given" to the #GdaDataSelect implementation using gda_data_select_take_row ().
+ * Each new #GdaRow created needs to be "given" to the #GdaDataSelect implementation using
+ * gda_data_select_take_row() because backward iterating is not supported.
  */
 static gboolean
 gda_sqlite_recordset_fetch_random (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaSqliteRecordset *imodel;
 
-	if (*prow)
-		return TRUE;
-
 	imodel = GDA_SQLITE_RECORDSET (model);
-	for (; imodel->priv->next_row_num <= rownum; ) {
-		*prow = fetch_next_sqlite_row (imodel, TRUE, error);
-		if (!*prow) {
-			/*if (GDA_DATA_SELECT (model)->advertized_nrows >= 0), it's not an error */
-			if ((GDA_DATA_SELECT (model)->advertized_nrows >= 0) && 
-			    (imodel->priv->next_row_num < rownum)) {
-				g_set_error (error, 0,
-					     GDA_DATA_MODEL_ROW_OUT_OF_RANGE_ERROR,
-					     _("Row %d not found"), rownum);
-			}
-			return FALSE;
-		}
-	}
-	if (! *prow) {
-		*prow = gda_data_select_get_stored_row (model, rownum);
-		if (!*prow)
-			return FALSE;
+	if (imodel->priv->next_row_num >= rownum) {
+		g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
+			     GDA_SERVER_PROVIDER_INTERNAL_ERROR, 
+			     "%s", _("Requested row could not be found"));
+		return TRUE;
 	}
+	for (*prow = fetch_next_sqlite_row (imodel, TRUE, error);
+	     *prow && (imodel->priv->next_row_num < rownum);
+	     *prow = fetch_next_sqlite_row (imodel, TRUE, error));
 
 	return TRUE;
 }
@@ -655,16 +695,29 @@ gda_sqlite_recordset_fetch_random (GdaDataSelect *model, GdaRow **prow, gint row
  * Create a new filled #GdaRow object for the next cursor row
  *
  * Each new #GdaRow created is referenced only by imodel->priv->tmp_row (the #GdaDataSelect implementation
- * never keeps a reference to it). Before a new #GdaRow gets created, the previous one, if set, is discarded.
+ * never keeps a reference to it).
+ * Before a new #GdaRow gets created, the previous one, if set, is discarded.
  */
 static gboolean
-gda_sqlite_recordset_fetch_next (GdaDataSelect *model, GdaRow **prow, G_GNUC_UNUSED gint rownum, GError **error)
+gda_sqlite_recordset_fetch_next (GdaDataSelect *model, GdaRow **prow, gint rownum, GError **error)
 {
 	GdaSqliteRecordset *imodel = (GdaSqliteRecordset*) model;
 
-	if (imodel->priv->tmp_row) 
+	if (imodel->priv->tmp_row) {
 		g_object_unref (imodel->priv->tmp_row);
-
+		imodel->priv->tmp_row = NULL;
+	}
+	if (imodel->priv->next_row_num != rownum) {
+		GError *lerror = NULL;
+		*prow = NULL;
+		g_set_error (&lerror, GDA_DATA_MODEL_ERROR,
+			     GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
+			     "%s", _("Can't set iterator on requested row"));
+		gda_data_select_add_exception (GDA_DATA_SELECT (model), lerror);
+		if (error)
+			g_propagate_error (error, g_error_copy (lerror));
+		return TRUE;
+	}
 	*prow = fetch_next_sqlite_row (imodel, FALSE, error);
 	imodel->priv->tmp_row = *prow;
 
