@@ -241,7 +241,10 @@ static sqlite3_module Module = {
 	virtualCommit,                /* xCommit - commit transaction */
 	virtualRollback,              /* xRollback - rollback transaction */
 	NULL,                         /* xFindFunction - function overloading */
-	virtualRename                 /* Rename - Notification that the table will be given a new name */
+	virtualRename,                /* Rename - Notification that the table will be given a new name */
+	NULL,                         /* xSavepoint */  
+	NULL,                         /* xRelease */
+	NULL                          /* xRollbackTo */
 };
 
 /*
@@ -277,7 +280,7 @@ handle_data_model_exception (sqlite3_vtab *pVtab, GdaDataModel *model)
 		if (pVtab->zErrMsg)
 			SQLITE3_CALL (sqlite3_free) (pVtab->zErrMsg);
 		pVtab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-			(e->message ? e->message : _("No detail"));
+			("%s", e->message ? e->message : _("No detail"));
 		if (fatal_error)
 			return SQLITE_ERROR;
 		else
@@ -530,11 +533,14 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 			type = "binary";
 		else if (gtype == G_TYPE_STRING)
 			type = "string";
-		else if ((gtype == G_TYPE_INT) || (gtype == G_TYPE_UINT) || 
-			 (gtype == G_TYPE_INT64) || (gtype == G_TYPE_UINT64) ||
-			 (gtype == GDA_TYPE_SHORT) || (gtype == GDA_TYPE_USHORT) ||
-			 (gtype == G_TYPE_LONG) || (gtype == G_TYPE_ULONG))
+		else if (gtype == G_TYPE_INT)
 			type = "integer";
+		else if (gtype == G_TYPE_UINT)
+			type = "unsigned integer";
+		else if ((gtype == G_TYPE_INT64) || (gtype == G_TYPE_LONG))
+			type = "int64";
+		else if ((gtype == G_TYPE_UINT64) || (gtype == G_TYPE_ULONG))
+			type = "uint64";
 		else if ((gtype == G_TYPE_DOUBLE) || (gtype == G_TYPE_FLOAT))
 			type = "real";
 		else if (gtype == G_TYPE_DATE)
@@ -543,7 +549,11 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 			type = "time";
 		else if (gtype == GDA_TYPE_TIMESTAMP)
 			type = "timestamp";
-		else 
+		else if (gtype == GDA_TYPE_SHORT)
+			type = "short";
+		else if (gtype == GDA_TYPE_USHORT)
+			type = "unsigned short";
+		else
 			type = "text";
 
 		g_string_append (sql, newcolname);
@@ -691,14 +701,18 @@ virtualColumn (sqlite3_vtab_cursor *cur, sqlite3_context *ctx, int i)
 
 	param = gda_data_model_iter_get_holder_for_field (cursor->iter, i);
 	if (!param) {
-		SQLITE3_CALL (sqlite3_result_error) (ctx, _("Column not found"), -1);
+		SQLITE3_CALL (sqlite3_result_text) (ctx, _("Column not found"), -1, SQLITE_TRANSIENT);
 		return SQLITE_EMPTY;
 	}
 	else {
 		const GValue *value;
+		GError *lerror = NULL;
 		value = gda_holder_get_value (param);
-
-		if (!value || gda_value_is_null (value))
+		if (! gda_holder_is_valid_e (param, &lerror)) {
+			g_hash_table_insert (error_blobs_hash, lerror, GINT_TO_POINTER (1));
+			SQLITE3_CALL (sqlite3_result_blob) (ctx, lerror, sizeof (GError), NULL);
+		}
+		else if (!value || gda_value_is_null (value))
 			SQLITE3_CALL (sqlite3_result_null) (ctx);
 		else  if (G_VALUE_TYPE (value) == G_TYPE_INT) 
 			SQLITE3_CALL (sqlite3_result_int) (ctx, g_value_get_int (value));
@@ -906,10 +920,31 @@ virtualFilter (sqlite3_vtab_cursor *pVtabCursor, int idxNum, const char *idxStr,
 	else
 		cursor->iter = gda_data_model_create_iter (vtable->td->real_model);
 
+	if (gda_data_model_iter_is_valid (cursor->iter) &&
+	    (gda_data_model_iter_get_row (cursor->iter) > 0)) {
+		/*g_print ("@%d, rewinding...", gda_data_model_iter_get_row (cursor->iter));*/
+		for (;gda_data_model_iter_move_prev (cursor->iter););
+		/*g_print ("done: @%d\n", gda_data_model_iter_get_row (cursor->iter));*/
+		if (gda_data_model_iter_is_valid (cursor->iter))
+			goto onerror;
+	}
+	if (! gda_data_model_iter_is_valid (cursor->iter)) {
+		gda_data_model_iter_move_next (cursor->iter);
+		if (! gda_data_model_iter_is_valid (cursor->iter))
+			goto onerror;
+	}
 	cursor->model = g_object_ref (vtable->td->real_model);
-	gda_data_model_iter_move_next (cursor->iter);
 
 	return handle_data_model_exception (pVtabCursor->pVtab, cursor->model);
+
+ onerror:
+	g_object_unref (cursor->iter);
+	cursor->iter = NULL;
+	if (pVtabCursor->pVtab->zErrMsg)
+			SQLITE3_CALL (sqlite3_free) (pVtabCursor->pVtab->zErrMsg);
+	pVtabCursor->pVtab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
+		("%s", _("Can't obtain an iterator located on the first row"));
+	return SQLITE_ERROR;
 }
 
 #ifdef GDA_DEBUG_VIRTUAL
