@@ -26,6 +26,8 @@
 #include <libgda-ui/gdaui-data-selector.h>
 #include "../support.h"
 #include "../browser-window.h"
+#include "../browser-stock-icons.h"
+#include "widget-overlay.h"
 #include <libgda/gda-data-model-extra.h>
 #ifdef HAVE_LDAP
 #include "../ldap-browser/ldap-browser-perspective.h"
@@ -45,6 +47,17 @@ static void ui_formgrid_get_property (GObject *object,
 				      GParamSpec *pspec);
 static void ui_formgrid_show (GtkWidget *widget);
 static BrowserConnection *get_browser_connection (UiFormGrid *formgrid);
+static void compute_modification_statements (UiFormGrid *formgrid, GdaDataModel *model);
+
+#define GRID_FLAGS (GDAUI_DATA_PROXY_INFO_CURRENT_ROW | GDAUI_DATA_PROXY_INFO_CHUNCK_CHANGE_BUTTONS)
+#define FORM_FLAGS (GDAUI_DATA_PROXY_INFO_CURRENT_ROW | GDAUI_DATA_PROXY_INFO_ROW_MOVE_BUTTONS)
+
+typedef enum {
+	MOD_INSERT,
+	MOD_UPDATE,
+	MOD_DELETE,
+	MOD_LAST
+} ModType;
 
 struct _UiFormGridPriv
 {
@@ -52,6 +65,8 @@ struct _UiFormGridPriv
 	GtkWidget   *raw_form;
 	GtkWidget   *raw_grid;
 	GtkWidget   *info;
+	GtkWidget   *overlay_form;
+	GtkWidget   *overlay_grid;
 	GtkWidget   *autoupdate_toggle;
 	gboolean     autoupdate;
 	gboolean     autoupdate_possible;
@@ -59,6 +74,13 @@ struct _UiFormGridPriv
 	
 	BrowserConnection *bcnc;
 	gboolean     scroll_form;
+
+	/* modifications to the data */
+	gboolean           compute_mod_stmt; /* if %TRUE, then the INSERT, UPDATE and DELETE
+					       * statements are automatically computed, see the PROP_AUTOMOD property */
+	gboolean           mod_stmt_auto_computed; /* %TRUE if mod_stmt[*] have automatically
+						    * been computed */
+	GdaStatement      *mod_stmt[MOD_LAST];
 };
 
 /* get a pointer to the parents to be able to call their destructor */
@@ -78,7 +100,8 @@ enum {
 	PROP_RAW_GRID,
 	PROP_RAW_FORM,
 	PROP_INFO,
-	PROP_SCROLL_FORM
+	PROP_SCROLL_FORM,
+	PROP_AUTOMOD
 };
 
 GType
@@ -100,7 +123,7 @@ ui_formgrid_get_type (void)
 			0
 		};		
 
-		type = g_type_register_static (GTK_TYPE_VBOX, "UiFormGrid", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "UiFormGrid", &info, 0);
 	}
 
 	return type;
@@ -144,6 +167,10 @@ ui_formgrid_class_init (UiFormGridClass *klass)
 					 g_param_spec_boolean ("scroll-form", NULL, NULL,
 							       FALSE,
 							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class, PROP_AUTOMOD,
+					 g_param_spec_boolean ("compute-mod-statements", NULL, NULL,
+							       FALSE,
+							       G_PARAM_WRITABLE | G_PARAM_CONSTRUCT));
 }
 
 static void
@@ -156,6 +183,14 @@ ui_formgrid_dispose (GObject *object)
                 if (formgrid->priv->bcnc)
                         g_object_unref (formgrid->priv->bcnc);
 
+		ModType mod;
+		for (mod = MOD_INSERT; mod < MOD_LAST; mod++) {
+			if (formgrid->priv->mod_stmt[mod]) {
+				g_object_unref (formgrid->priv->mod_stmt[mod]);
+				formgrid->priv->mod_stmt[mod] = NULL;
+			}
+		}
+
                 g_free (formgrid->priv);
                 formgrid->priv = NULL;
         }
@@ -168,25 +203,78 @@ static void
 ui_formgrid_show (GtkWidget *widget)
 {
 	UiFormGrid *formgrid;
+	GtkWidget *ovl, *packed;
 	formgrid = UI_FORMGRID (widget);
 
-	/* finalize packing */
-	if (formgrid->priv->scroll_form) {
+	if (! formgrid->priv->overlay_grid) {
+		/* finalize packing */
 		GtkWidget *sw, *vp;
 		sw = gtk_scrolled_window_new (NULL, NULL);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-						GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-		gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
-		vp = gtk_viewport_new (NULL, NULL);
-		gtk_container_add (GTK_CONTAINER (sw), vp);
-		gtk_viewport_set_shadow_type (GTK_VIEWPORT (vp), GTK_SHADOW_NONE);
-		gtk_container_add (GTK_CONTAINER (vp), formgrid->priv->raw_form);
-		gtk_notebook_append_page (GTK_NOTEBOOK (formgrid->priv->nb), sw, NULL);
+						GTK_POLICY_AUTOMATIC,
+						GTK_POLICY_AUTOMATIC);
+		gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
+						     GTK_SHADOW_NONE);
+		gtk_container_add (GTK_CONTAINER (sw), formgrid->priv->raw_grid);
 		gtk_widget_show_all (sw);
+		packed = sw;
+
+		/* overlay */
+		ovl = widget_overlay_new ();
+		formgrid->priv->overlay_grid = ovl;
+		g_object_set (G_OBJECT (ovl), "add-scale", TRUE, NULL);
+		g_object_set (G_OBJECT (ovl), "add-scale", FALSE, NULL);
+		gtk_container_add (GTK_CONTAINER (ovl), packed);
+		widget_overlay_set_child_props (WIDGET_OVERLAY (ovl), packed,
+						WIDGET_OVERLAY_CHILD_HALIGN,
+						WIDGET_OVERLAY_ALIGN_FILL,
+						WIDGET_OVERLAY_CHILD_VALIGN,
+						WIDGET_OVERLAY_ALIGN_FILL,
+						WIDGET_OVERLAY_CHILD_SCALE, .9,
+						-1);
+		gtk_widget_show (ovl);
+		gtk_notebook_append_page (GTK_NOTEBOOK (formgrid->priv->nb), ovl, NULL);
 	}
-	else {
-		gtk_notebook_append_page (GTK_NOTEBOOK (formgrid->priv->nb), formgrid->priv->raw_form, NULL);
-		gtk_widget_show (formgrid->priv->raw_form);
+
+	if (! formgrid->priv->overlay_form) {
+		/* finalize packing */
+		if (formgrid->priv->scroll_form) {
+			GtkWidget *sw, *vp;
+			sw = gtk_scrolled_window_new (NULL, NULL);
+			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
+							GTK_POLICY_AUTOMATIC,
+							GTK_POLICY_AUTOMATIC);
+			gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
+							     GTK_SHADOW_NONE);
+			vp = gtk_viewport_new (NULL, NULL);
+			gtk_container_add (GTK_CONTAINER (sw), vp);
+			gtk_viewport_set_shadow_type (GTK_VIEWPORT (vp), GTK_SHADOW_NONE);
+			gtk_container_add (GTK_CONTAINER (vp), formgrid->priv->raw_form);
+			gtk_widget_show_all (sw);
+			packed = sw;
+		}
+		else {
+			gtk_widget_show (formgrid->priv->raw_form);
+			packed = formgrid->priv->raw_form;
+		}
+		
+		/* overlay */
+		ovl = widget_overlay_new ();
+		formgrid->priv->overlay_form = ovl;
+		g_object_set (G_OBJECT (ovl), "add-scale", TRUE, NULL);
+		g_object_set (G_OBJECT (ovl), "add-scale", FALSE, NULL);
+		gtk_container_add (GTK_CONTAINER (ovl), packed);
+		widget_overlay_set_child_props (WIDGET_OVERLAY (ovl), packed,
+						WIDGET_OVERLAY_CHILD_HALIGN,
+						WIDGET_OVERLAY_ALIGN_FILL,
+						WIDGET_OVERLAY_CHILD_VALIGN,
+						WIDGET_OVERLAY_ALIGN_FILL,
+						WIDGET_OVERLAY_CHILD_SCALE, 1.,
+						-1);
+		gtk_widget_show (ovl);
+		gtk_notebook_append_page (GTK_NOTEBOOK (formgrid->priv->nb), ovl, NULL);
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (formgrid->priv->nb),
+					       0);
 	}
 
 	((GtkWidgetClass *)parent_class)->show (widget);
@@ -195,8 +283,9 @@ ui_formgrid_show (GtkWidget *widget)
 }
 
 static void form_grid_autoupdate_cb (GtkToggleButton *button, UiFormGrid *formgrid);
-static void form_grid_toggled_cb (GtkToggleButton *button, UiFormGrid *formgrid);
+static void form_grid_toggled_cb (GtkToggleAction *action, UiFormGrid *formgrid);
 static void form_grid_populate_popup_cb (GtkWidget *wid, GtkMenu *menu, UiFormGrid *formgrid);
+static void selection_changed_cb (GdauiDataSelector *sel, UiFormGrid *formgrid);
 
 static void
 ui_formgrid_init (UiFormGrid *formgrid)
@@ -207,11 +296,14 @@ ui_formgrid_init (UiFormGrid *formgrid)
 	formgrid->priv = g_new0 (UiFormGridPriv, 1);
 	formgrid->priv->raw_grid = NULL;
 	formgrid->priv->info = NULL;
-	formgrid->priv->flags = GDAUI_DATA_PROXY_INFO_CURRENT_ROW;
+	formgrid->priv->flags = GDAUI_DATA_PROXY_INFO_CURRENT_ROW | GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS;
 	formgrid->priv->bcnc = NULL;
 	formgrid->priv->autoupdate = TRUE;
 	formgrid->priv->autoupdate_possible = FALSE;
 	formgrid->priv->scroll_form = FALSE;
+	formgrid->priv->compute_mod_stmt = FALSE;
+
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (formgrid), GTK_ORIENTATION_VERTICAL);
 
 	/* notebook */
 	formgrid->priv->nb = gtk_notebook_new ();
@@ -220,43 +312,23 @@ ui_formgrid_init (UiFormGrid *formgrid)
 	gtk_box_pack_start (GTK_BOX (formgrid), formgrid->priv->nb, TRUE, TRUE, 0);
 	gtk_widget_show (formgrid->priv->nb);
 
-	/* grid on 1st page of notebook */
-	sw = gtk_scrolled_window_new (NULL, NULL);
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_NONE);
-	gtk_notebook_append_page (GTK_NOTEBOOK (formgrid->priv->nb), sw, NULL);
-	gtk_widget_show (sw);
-
+	/* grid on 1st page of notebook, not added there */
 	formgrid->priv->raw_grid = gdaui_raw_grid_new (NULL);
 	gdaui_data_proxy_column_show_actions (GDAUI_DATA_PROXY (formgrid->priv->raw_grid), -1, FALSE);
-	gtk_container_add (GTK_CONTAINER (sw), formgrid->priv->raw_grid);
-	gtk_widget_show (formgrid->priv->raw_grid);
 	g_signal_connect (formgrid->priv->raw_grid, "populate-popup",
 			  G_CALLBACK (form_grid_populate_popup_cb), formgrid);
 
-	/* form on the 2nd page of the notebook */
+	/* form on the 2nd page of the notebook, not added there */
 	formgrid->priv->raw_form = gdaui_raw_form_new (NULL);
 	gdaui_data_proxy_column_show_actions (GDAUI_DATA_PROXY (formgrid->priv->raw_form), -1, FALSE);
 	g_signal_connect (formgrid->priv->raw_form, "populate-popup",
 			  G_CALLBACK (form_grid_populate_popup_cb), formgrid);
 
 	/* info widget and toggle button at last */
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_box_pack_start (GTK_BOX (formgrid), hbox, FALSE, TRUE, 0);
 	gtk_widget_show (hbox);
 
-	/* button to toggle between grid and form mode */
-	button = gtk_toggle_button_new ();
-	GdkPixbuf *pixbuf = browser_get_pixbuf_icon (BROWSER_ICON_GRID);
-	gtk_button_set_image (GTK_BUTTON (button), gtk_image_new_from_pixbuf (pixbuf));
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, TRUE, 0);
-	gtk_widget_show (button);
-	g_signal_connect (G_OBJECT (button), "toggled",
-			  G_CALLBACK (form_grid_toggled_cb), formgrid);
-	gtk_widget_set_tooltip_text (button, _("Toggle between grid and form presentations"));
-
 	/* button to toggle between auto update and not */
 	button = gtk_toggle_button_new ();
 	GtkWidget *img = gtk_image_new_from_stock (GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU);
@@ -269,12 +341,90 @@ ui_formgrid_init (UiFormGrid *formgrid)
 			  G_CALLBACK (form_grid_autoupdate_cb), formgrid);
 	gtk_widget_set_tooltip_text (button, _("Enable or disable auto update of data"));
 
+	/* Proxy info */
 	formgrid->priv->info = gdaui_data_proxy_info_new (GDAUI_DATA_PROXY (formgrid->priv->raw_grid), 
 							  formgrid->priv->flags |
 							  GDAUI_DATA_PROXY_INFO_CURRENT_ROW |
 							  GDAUI_DATA_PROXY_INFO_CHUNCK_CHANGE_BUTTONS);
+
+	GtkUIManager *uimanager;
+	GtkActionGroup *agroup;
+	GtkAction *action;
+	guint mid;
+	
+	g_object_get (G_OBJECT (formgrid->priv->info), "ui-manager", &uimanager, NULL);
+	agroup = gtk_action_group_new ("FormGrid");
+	gtk_action_group_set_translation_domain (agroup, GETTEXT_PACKAGE);
+
+	action = GTK_ACTION (gtk_toggle_action_new ("FGToggle", "FGToggle",
+						    _("Toggle between grid and form presentations"),
+						    BROWSER_STOCK_GRID));
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+	gtk_action_group_add_action (agroup, action);
+	g_signal_connect (G_OBJECT (action), "toggled",
+			  G_CALLBACK (form_grid_toggled_cb), formgrid);
+	g_object_unref (action);
+	gtk_ui_manager_insert_action_group (uimanager, agroup, 0);
+	g_object_unref (agroup);
+
+	mid = gtk_ui_manager_new_merge_id (uimanager);
+	gtk_ui_manager_add_ui (uimanager, mid, "/ToolBar", "FGToggle", "FGToggle",
+			       GTK_UI_MANAGER_AUTO, TRUE);
+	gtk_ui_manager_ensure_update (uimanager);
 	gtk_box_pack_start (GTK_BOX (hbox), formgrid->priv->info, TRUE, TRUE, 0);
 	gtk_widget_show (formgrid->priv->info);
+	g_object_unref (G_OBJECT (uimanager));
+
+
+	/*gchar *tmp;
+	g_object_get (uimanager, "ui", &tmp, NULL);
+	g_print ("==>[%s]\n", tmp);
+	g_free (tmp);
+	*/
+
+	/* keep data in sync */
+	g_signal_connect (formgrid->priv->raw_grid, "selection-changed",
+			  G_CALLBACK (selection_changed_cb), formgrid);
+	g_signal_connect (formgrid->priv->raw_form, "selection-changed",
+			  G_CALLBACK (selection_changed_cb), formgrid);
+}
+
+/**
+ * ui_formgrid_get_ui_manager:
+ *
+ * Returns: (transfer none): the #GtkUIManager
+ */
+GtkUIManager *
+ui_formgrid_get_ui_manager (UiFormGrid *formgrid)
+{
+	GtkUIManager *uimanager;
+	g_return_val_if_fail (UI_IS_FORMGRID (formgrid), NULL);
+	g_object_get (G_OBJECT (formgrid->priv->info), "ui-manager", &uimanager, NULL);
+	g_object_unref (uimanager);
+	return uimanager;
+}
+
+static void
+selection_changed_cb (GdauiDataSelector *sel, UiFormGrid *formgrid)
+{
+	GdaDataModelIter *iter;
+	GdauiDataSelector *tosel;
+	gint row;
+	if (sel == (GdauiDataSelector*) formgrid->priv->raw_grid)
+		tosel = (GdauiDataSelector*) formgrid->priv->raw_form;
+	else
+		tosel = (GdauiDataSelector*) formgrid->priv->raw_grid;
+
+	iter = gdaui_data_selector_get_data_set (sel);
+	g_assert (iter);
+	row = gda_data_model_iter_get_row (iter);
+	/*g_print ("Moved %s to row %d\n", sel == (GdauiDataSelector*) formgrid->priv->raw_grid ? "grid" : "form", row);*/
+	iter = gdaui_data_selector_get_data_set (tosel);
+	if (iter) {
+		g_signal_handlers_block_by_func (tosel, G_CALLBACK (selection_changed_cb), formgrid);
+		gda_data_model_iter_move_to_row (iter, row >= 0 ? row : 0);
+		g_signal_handlers_unblock_by_func (tosel, G_CALLBACK (selection_changed_cb), formgrid);
+	}
 }
 
 static void
@@ -284,52 +434,26 @@ form_grid_autoupdate_cb (GtkToggleButton *button, UiFormGrid *formgrid)
 }
 
 static void
-form_grid_toggled_cb (GtkToggleButton *button, UiFormGrid *formgrid)
+form_grid_toggled_cb (GtkToggleAction *action, UiFormGrid *formgrid)
 {
-	GdaDataModelIter *iter;
-	gint row;
-
-	if (!gtk_toggle_button_get_active (button)) {
+	if (!gtk_toggle_action_get_active (action)) {
 		/* switch to form  view */
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (formgrid->priv->nb), 1);
 		g_object_set (G_OBJECT (formgrid->priv->info),
 			      "data-proxy", formgrid->priv->raw_form,
-			      "flags", formgrid->priv->flags | GDAUI_DATA_PROXY_INFO_CURRENT_ROW |
-			      GDAUI_DATA_PROXY_INFO_ROW_MOVE_BUTTONS
-			      /*GDAUI_DATA_PROXY_INFO_CURRENT_ROW |
-				GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS |
-				GDAUI_DATA_PROXY_INFO_ROW_MOVE_BUTTONS*/, NULL);
+			      "flags", formgrid->priv->flags | FORM_FLAGS, NULL);
 
-		GdkPixbuf *pixbuf = browser_get_pixbuf_icon (BROWSER_ICON_FORM);
-		gtk_button_set_image (GTK_BUTTON (button), gtk_image_new_from_pixbuf (pixbuf));
-
-		iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (formgrid->priv->raw_grid));
-		if (iter)
-			row = gda_data_model_iter_get_row (iter);
-		iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (formgrid->priv->raw_form));
+		g_object_set (G_OBJECT (action), "stock-id", BROWSER_STOCK_FORM, NULL);
 	}
 	else {
 		/* switch to grid view */
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (formgrid->priv->nb), 0);
 		g_object_set (G_OBJECT (formgrid->priv->info),
 			      "data-proxy", formgrid->priv->raw_grid,
-			      "flags", formgrid->priv->flags | GDAUI_DATA_PROXY_INFO_CURRENT_ROW |
-			      GDAUI_DATA_PROXY_INFO_CHUNCK_CHANGE_BUTTONS
-			      /*GDAUI_DATA_PROXY_INFO_CURRENT_ROW |
-				GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS |
-				GDAUI_DATA_PROXY_INFO_CHUNCK_CHANGE_BUTTONS*/, NULL);
+			      "flags", formgrid->priv->flags | GRID_FLAGS, NULL);
 
-		GdkPixbuf *pixbuf = browser_get_pixbuf_icon (BROWSER_ICON_GRID);
-		gtk_button_set_image (GTK_BUTTON (button), gtk_image_new_from_pixbuf (pixbuf));
-
-		iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (formgrid->priv->raw_form));
-		if (iter)
-			row = gda_data_model_iter_get_row (iter);
-		iter = gdaui_data_selector_get_data_set (GDAUI_DATA_SELECTOR (formgrid->priv->raw_grid));
+		g_object_set (G_OBJECT (action), "stock-id", BROWSER_STOCK_GRID, NULL);
 	}
-
-	if (iter)
-		gda_data_model_iter_move_to_row (iter, row >= 0 ? row : 0);
 }
 
 static BrowserConnection *
@@ -352,9 +476,11 @@ static void execute_action_mitem_cb (GtkMenuItem *menuitem, UiFormGrid *formgrid
 #ifdef HAVE_LDAP
 static void ldap_view_dn_mitem_cb (GtkMenuItem *menuitem, UiFormGrid *formgrid);
 #endif
+static void zoom_form_mitem_cb (GtkCheckMenuItem *checkmenuitem, UiFormGrid *formgrid);
+static void zoom_grid_mitem_cb (GtkCheckMenuItem *checkmenuitem, UiFormGrid *formgrid);
 
 static void
-form_grid_populate_popup_cb (G_GNUC_UNUSED GtkWidget *wid, GtkMenu *menu, UiFormGrid *formgrid)
+form_grid_populate_popup_cb (GtkWidget *wid, GtkMenu *menu, UiFormGrid *formgrid)
 {
 	/* add actions to execute to menu */
 	GdaDataModelIter *iter;
@@ -430,6 +556,43 @@ form_grid_populate_popup_cb (G_GNUC_UNUSED GtkWidget *wid, GtkMenu *menu, UiForm
 		}
 	}
 #endif
+
+	if (wid == formgrid->priv->raw_form) {
+		GtkWidget *mitem;
+		gboolean add_scale;
+		g_object_get (G_OBJECT (formgrid->priv->overlay_form), "add-scale", &add_scale, NULL);
+		mitem = gtk_check_menu_item_new_with_label (_("Zoom..."));
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mitem), add_scale);
+		gtk_widget_show (mitem);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), mitem);
+		g_signal_connect (mitem, "toggled",
+				  G_CALLBACK (zoom_form_mitem_cb), formgrid);
+	}
+	else if (wid == formgrid->priv->raw_grid) {
+		GtkWidget *mitem;
+		gboolean add_scale;
+		g_object_get (G_OBJECT (formgrid->priv->overlay_grid), "add-scale", &add_scale, NULL);
+		mitem = gtk_check_menu_item_new_with_label (_("Zoom..."));
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mitem), add_scale);
+		gtk_widget_show (mitem);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), mitem);
+		g_signal_connect (mitem, "toggled",
+				  G_CALLBACK (zoom_grid_mitem_cb), formgrid);
+	}
+}
+
+static void
+zoom_form_mitem_cb (GtkCheckMenuItem *checkmenuitem, UiFormGrid *formgrid)
+{
+	g_object_set (G_OBJECT (formgrid->priv->overlay_form), "add-scale",
+		      gtk_check_menu_item_get_active (checkmenuitem), NULL);
+}
+
+static void
+zoom_grid_mitem_cb (GtkCheckMenuItem *checkmenuitem, UiFormGrid *formgrid)
+{
+	g_object_set (G_OBJECT (formgrid->priv->overlay_grid), "add-scale",
+		      gtk_check_menu_item_get_active (checkmenuitem), NULL);
 }
 
 typedef struct {
@@ -442,6 +605,9 @@ typedef struct {
 	guint          exec_id;
 	guint          timer_id;
 } ActionExecutedData;
+
+static void action_executed_holder_changed_cb (G_GNUC_UNUSED GdaSet *params, G_GNUC_UNUSED GdaHolder *holder,
+					       ActionExecutedData *aed);
 static void
 action_executed_data_free (ActionExecutedData *data)
 {
@@ -450,7 +616,12 @@ action_executed_data_free (ActionExecutedData *data)
 		g_object_unref ((GObject*) data->formgrid);
 	g_free (data->name);
 	g_object_unref ((GObject*) data->stmt);
-	g_object_unref ((GObject*) data->params);
+
+	if (data->params) {
+		g_signal_handlers_disconnect_by_func (data->params,
+						      G_CALLBACK (action_executed_holder_changed_cb), data);
+		g_object_unref ((GObject*) data->params);
+	}
 	if (data->model)
 		g_object_unref ((GObject*) data->model);
 	if (data->timer_id)
@@ -542,6 +713,7 @@ statement_executed_cb (G_GNUC_UNUSED BrowserConnection *bcnc,
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (aed->formgrid));
 	g_object_unref (aed->formgrid);
 	aed->formgrid = NULL;
+
         if (error)
                 browser_show_error (GTK_WINDOW (toplevel),
                                     _("Error executing query:\n%s"),
@@ -595,7 +767,9 @@ statement_executed_cb (G_GNUC_UNUSED BrowserConnection *bcnc,
 				  G_CALLBACK (gtk_widget_destroy), NULL);
 		g_signal_connect (dialog, "close",
 				  G_CALLBACK (gtk_widget_destroy), NULL);
-		aed = NULL;
+		g_object_set_data_full (G_OBJECT (dialog), "aed", aed,
+					(GDestroyNotify) action_executed_data_free);
+		aed = NULL; /* don't free it yet */
 	}
         else if (BROWSER_IS_WINDOW (toplevel)) {
 		browser_window_show_notice_printf (BROWSER_WINDOW (toplevel),
@@ -677,7 +851,7 @@ static void ldap_view_dn_mitem_cb (GtkMenuItem *menuitem, UiFormGrid *formgrid)
 #endif
 
 static void
-proxy_changed_cb (GdauiDataProxy *dp, GdaDataProxy *proxy, UiFormGrid *formgrid)
+proxy_changed_cb (G_GNUC_UNUSED GdauiDataProxy *dp, G_GNUC_UNUSED GdaDataProxy *proxy, UiFormGrid *formgrid)
 {
 	g_signal_emit (formgrid, ui_formgrid_signals [DATA_SET_CHANGED], 0);
 }
@@ -708,17 +882,20 @@ ui_formgrid_new (GdaDataModel *model, gboolean scroll_form, GdauiDataProxyInfoFl
 	proxy = gdaui_data_proxy_get_proxy (GDAUI_DATA_PROXY (formgrid->priv->raw_grid));
 	g_object_set (formgrid->priv->raw_form, "model", proxy, NULL);
 	gdaui_data_proxy_set_write_mode (GDAUI_DATA_PROXY (formgrid->priv->raw_form),
-					 GDAUI_DATA_PROXY_WRITE_ON_ROW_CHANGE);
+					 GDAUI_DATA_PROXY_WRITE_ON_DEMAND);
 	g_object_set (G_OBJECT (formgrid->priv->info),
-		      "flags", formgrid->priv->flags | GDAUI_DATA_PROXY_INFO_CURRENT_ROW |
-		      GDAUI_DATA_PROXY_INFO_CHUNCK_CHANGE_BUTTONS, NULL);
+		      "flags", formgrid->priv->flags | GRID_FLAGS, NULL);
 
 	g_signal_connect (formgrid->priv->raw_grid, "proxy-changed",
 			  G_CALLBACK (proxy_changed_cb), formgrid);
 
 	/* no more than 300 rows at a time */
-	if (model)
+	if (model) {
 		gda_data_proxy_set_sample_size (proxy, 300);
+		if (flags & GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS)
+			g_object_set (G_OBJECT (formgrid), "compute-mod-statements", TRUE, NULL);
+	}
+
 
 	return (GtkWidget *) formgrid;
 }
@@ -726,7 +903,7 @@ ui_formgrid_new (GdaDataModel *model, gboolean scroll_form, GdauiDataProxyInfoFl
 /**
  * ui_formgrid_handle_user_prefs
  * @formgrid: a #UiFormGrid widget
- * @bcnc: a #BrowserConnection, or %NULL to let @formgrid determine it itself
+ * @bcnc: (allow-none): a #BrowserConnection, or %NULL to let @formgrid determine it itself
  * @stmt: the #GdaStatement which has been executed to produce the #GdaDataModel displayed in @formgrid
  *
  * Takes into account the UI preferences of the user
@@ -809,6 +986,31 @@ ui_formgrid_set_property (GObject *object,
 	case PROP_SCROLL_FORM:
 		formgrid->priv->scroll_form = g_value_get_boolean (value);
 		break;
+	case PROP_AUTOMOD:
+		formgrid->priv->compute_mod_stmt = g_value_get_boolean (value);
+		if (formgrid->priv->mod_stmt_auto_computed && !formgrid->priv->compute_mod_stmt) {
+			/* clean up the mod stmt */
+			ModType mod;
+			GdaDataModel *model;
+			g_object_get (formgrid->priv->raw_grid, "model", &model, NULL);
+			for (mod = MOD_INSERT; mod < MOD_LAST; mod++) {
+				if (! formgrid->priv->mod_stmt[mod])
+					continue;
+				g_object_unref (formgrid->priv->mod_stmt [mod]);
+			}
+			g_object_set (model, "delete-stmt", NULL,
+				      "insert-stmt", NULL, NULL,
+				      "update-stmt", NULL, NULL);
+			g_object_unref (model);
+			formgrid->priv->mod_stmt_auto_computed = FALSE;
+		}
+		else if (!formgrid->priv->mod_stmt_auto_computed && formgrid->priv->compute_mod_stmt) {
+			GdaDataModel *model;
+			g_object_get (formgrid->priv->raw_grid, "model", &model, NULL);
+			compute_modification_statements (formgrid, model);
+			g_object_unref (model);
+		}
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -835,6 +1037,9 @@ ui_formgrid_get_property (GObject *object,
 	case PROP_INFO:
 		g_value_set_object (value, formgrid->priv->info);
 		break;
+	case PROP_AUTOMOD:
+		g_value_set_boolean (value, formgrid->priv->compute_mod_stmt);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -921,7 +1126,7 @@ ui_formgrid_get_grid_widget (UiFormGrid *formgrid)
 /**
  * ui_formgrid_set_connection
  * @formgrid: a #UiFormGrid widget
- * @bcnc: a #BrowserConnection, or %NULL
+ * @bcnc: (allow-none): a #BrowserConnection, or %NULL
  *
  * Tells @formgrid to use @bcnc as connection when actions have to be executed
  */
@@ -938,3 +1143,49 @@ ui_formgrid_set_connection (UiFormGrid *formgrid, BrowserConnection *bcnc)
 	if (bcnc)
 		formgrid->priv->bcnc = g_object_ref (bcnc);
 }
+
+static void
+compute_modification_statements (UiFormGrid *formgrid, GdaDataModel *model)
+{
+	/* clear existing modification statements */
+	ModType mod;
+	for (mod = MOD_INSERT; mod < MOD_LAST; mod++) {
+		if (formgrid->priv->mod_stmt[mod]) {
+			g_object_unref (formgrid->priv->mod_stmt[mod]);
+			formgrid->priv->mod_stmt[mod] = NULL;
+		}
+	}
+
+	if (!model || !GDA_IS_DATA_SELECT (model))
+		return;
+
+	if (! formgrid->priv->compute_mod_stmt)
+		return;
+
+	g_print ("Computing MOD STMTs\n");
+	gda_data_select_compute_modification_statements_ext (GDA_DATA_SELECT (model),
+							     GDA_DATA_SELECT_COND_ALL_COLUMNS, NULL);
+
+	formgrid->priv->mod_stmt_auto_computed = TRUE;
+	g_object_get (model,
+		      "insert-stmt", &formgrid->priv->mod_stmt[MOD_INSERT],
+		      "update-stmt", &formgrid->priv->mod_stmt[MOD_UPDATE],
+		      "delete-stmt", &formgrid->priv->mod_stmt[MOD_DELETE], NULL);
+
+	for (mod = MOD_INSERT; mod < MOD_LAST; mod++) {
+		if (formgrid->priv->mod_stmt[mod]) {
+			gchar *sql;
+			GError *lerror = NULL;
+			sql = gda_statement_to_sql_extended (formgrid->priv->mod_stmt[mod], NULL, NULL,
+							     GDA_STATEMENT_SQL_PARAMS_LONG, NULL, &lerror);
+			g_print ("STMT[%d] = [%s]", mod, sql ? sql : "ERR");
+			if (!sql)
+				g_print (" --- %s", lerror && lerror->message ? lerror->message : "No detail");
+			g_clear_error (&lerror);
+			g_print ("\n");
+			g_free (sql);
+		}
+		else
+			g_print ("STMT[%d] = ---\n", mod);
+	}
+}
